#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/netdevice.h>
#include <linux/sysctl.h>
#include "rtmp_comm.h"
#include "rt_config.h"



NDIS_STATUS MyMiniportMMRequest(
	IN RTMP_ADAPTER *pAd,
	IN UCHAR QueIdx,
	IN UCHAR *pData,
	IN UINT Length)
{
	PNDIS_PACKET pPacket;
	NDIS_STATUS Status = NDIS_STATUS_SUCCESS;
	ULONG FreeNum;
	UINT8 TXWISize = pAd->chipCap.TXWISize;
	UCHAR rtmpHwHdr[40];
	BOOLEAN bUseDataQ = FALSE, FlgDataQForce = FALSE, FlgIsLocked = FALSE;
	int retryCnt = 0, hw_len = TXINFO_SIZE + TXWISize + TSO_SIZE;

	bUseDataQ = FALSE;

	/* Check Free priority queue*/
	/* Since we use PBF Queue2 for management frame.  Its corresponding DMA ring should be using TxRing.*/
	{
		FreeNum = GET_MGMTRING_FREENO(pAd);
	}

	if ((FreeNum > 0))
	{
		/* We need to reserve space for rtmp hardware header. i.e., TxWI for RT2860 and TxInfo+TxWI for RT2870*/
		NdisZeroMemory(&rtmpHwHdr, hw_len);
		Status = RTMPAllocateNdisPacket(pAd, &pPacket, (PUCHAR)&rtmpHwHdr, hw_len, pData, Length);
//		pPacket = RTMP_AllocateFragPacketBuffer( 0, hw_len );
		if (Status != NDIS_STATUS_SUCCESS)
		{
			DBGPRINT(RT_DEBUG_WARN, ("MiniportMMRequest (error:: can't allocate NDIS PACKET)\n"));
			return -5;
		}
		Status = MlmeHardTransmit(pAd, QueIdx, pPacket, FlgDataQForce, FlgIsLocked);
		if (Status == NDIS_STATUS_SUCCESS)
			retryCnt = 0;
		else
			RELEASE_NDIS_PACKET(pAd, pPacket, Status);
	}
	else
	{
		pAd->RalinkCounters.MgmtRingFullCount++;
		DBGPRINT(RT_DEBUG_ERROR, ("Qidx(%d), not enough space in MgmtRing, MgmtRingFullCount=%ld!\n",
									QueIdx, pAd->RalinkCounters.MgmtRingFullCount));
	}

	

	return Status;
}




// 0x8B2E = 
int rt_ioctl_user_iface(struct net_device *dev,
		       struct iw_request_info *info,
		       struct iw_param *rts, char *extra)
{
	int to_copy;
	uint8_t gbbuff[2048];
	struct iw_point * rt = (struct iw_point *)rts;
	IN RTMP_ADAPTER *pAd = NULL;
//	RT_CMD_STA_IOCTL_SECURITY IoctlSec, *pIoctlSec = &IoctlSec;
	GET_PAD_FROM_NET_DEV(pAd, dev);

//	printk( "&*& User Interface %p %p (%d %d) %p %p\n", dev, info, info->cmd, info->flags, rts, extra );
//	printk( "-- %p %04x %d\n", rt->pointer, rt->flags, rt->length );
	to_copy = rt->length;
	if( to_copy > sizeof( gbbuff ) )
	{
		printk( "Error: Buffer too big.\n" );
		return -1;
	}
	copy_from_user( gbbuff, rt->pointer, to_copy );
	//DO STUFF HERE!!!

	//printk( "RATE INFO: %d\n", pAd->CommonCfg.MlmeTransmit.field.MCS );
	//pAd->CommonCfg.MlmeTransmit.field.MCS = 12;   //Various
	//pAd->CommonCfg.MlmeTransmit.field.MODE = 12;  //MODE_OFDM = 1, MODE_CCK = 0;
	pAd->CommonCfg.MlmeTransmit.field.MODE = rt->flags >> 12;
	pAd->CommonCfg.MlmeTransmit.field.MCS = rt->flags & 0xff;
	MyMiniportMMRequest(pAd, 1, gbbuff, to_copy ); //from cmm_data.c


	{
/*
		NDIS_STATUS NStatus;
		PUCHAR pOutBuffer = NULL;
		ULONG FrameLen = 0;
		int i;

		NStatus = MlmeAllocateMemory(pAd, &pOutBuffer);	//Get an unused nonpaged memory 
		if (NStatus != NDIS_STATUS_SUCCESS) {
			printk( "Error: Can't allocate page.\n" );
			return -8;
		}

		for( i = 0; i < to_copy; i++ )
		{
			pOutBuffer[i] = gbbuff[i];
		}
		FrameLen = to_copy;

/*
		HEADER_802_11 DisassocHdr;
		USHORT Reason = 5;

		MgtMacHeaderInit(pAd, &DisassocHdr, SUBTYPE_DISASSOC, 0, "000000", "111111" );

		MakeOutgoingFrame(pOutBuffer, &FrameLen,
				  sizeof (HEADER_802_11), &DisassocHdr,
				  2, &Reason, END_OF_ARGS);
		printk( "FRAME %d: ", FrameLen );
		for( i = 0; i < FrameLen; i++ )
		{
			printk( "0x%02x, ", pOutBuffer[i] );
		}
		printk( "\n" );
*/
//		MyMiniportMMRequest(pAd, 0, gbbuff, to_copy ); //from cmm_data.c
//		MlmeFreeMemory(pAd, pOutBuffer);
	}






//	printk( gbbuff );
	return 0;
}  

void user_iface_get( void * pAd, RTMP_IOCTL_INPUT_STRUCT * wrq, struct net_device * net_dev )
{
	//NOT USED.
	printk( "!!! User Interface %p %p %p\n", pAd, wrq, net_dev );
}


int setup_user_iface(void * pAd)
{
	printk( "STOP\n" );
}

void stop_user_iface(void * pAd)
{
	printk( "START\n" );
}

